﻿Public Class Form1
    ' The bitmap.
    Private Bm As Bitmap

    ' Parameters
    Private IterationsPerPixel As Integer
    Private H As Single

    ' Make a bitmap of the appropriate size.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        picCanvas.ClientSize = New Size(CInt(Dxmax), CInt(Dymax))

        Bm = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        picCanvas.Image = Bm
    End Sub

    ' Plot a bunch of points.
    Private Sub btnPlotAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPlotAll.Click
        GetParameters()

        ' Make a new bitmap.
        Bm = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        picCanvas.Image = Bm

        ' Plot a series for each point.
        Dim dx As Integer = Integer.Parse(txtDx.Text)
        For x As Integer = 0 To Bm.Width - 1 Step dx
            For y As Integer = 0 To Bm.Height - 1 Step dx
                PlotPoints(Bm, H, x, y, IterationsPerPixel)
            Next y
            picCanvas.Refresh()
        Next x
        picCanvas.Refresh()
    End Sub

    ' Clear.
    Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click
        ' Make a new bitmap.
        Bm = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        picCanvas.Image = Bm
        picCanvas.Refresh()
    End Sub

    ' Plot points for the clicked pixel.
    Private Sub picCanvas_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picCanvas.MouseClick
        GetParameters()

        ' Plot the point's series.
        PlotPoints(Bm, H, e.X, e.Y, IterationsPerPixel)
        picCanvas.Refresh()
    End Sub

    ' Get the parameters.
    Private Sub GetParameters()
        IterationsPerPixel = Integer.Parse(txtIterationsPerPixel.Text)
        H = Single.Parse(txtH.Text)
    End Sub

    ' Plot points for a pixel using the equations:
    '      x(n + 1) = x(n) - h * Sin(y(n) + Tan(3 * y(n)))
    '      y(n + 1) = y(n) - h * Sin(x(n) + Tan(3 * x(n)))
    Private Sub PlotPoints(ByVal bm As Bitmap, ByVal h As Single, ByVal pix_x As Integer, ByVal pix_y As Integer, ByVal iterations As Integer)
        ' Convert the first point to world coordinates.
        Dim wx, wy As Single
        DeviceToWorld(pix_x, pix_y, wx, wy)

        ' Plot points.
        bm.SetPixel(pix_x, pix_y, Color.Blue)
        For i As Integer = 1 To iterations
            Dim new_x As Single = CSng(wx - h * Math.Sin(wy + Math.Tan(3 * wy)))
            Dim new_y As Single = CSng(wy - h * Math.Sin(wx + Math.Tan(3 * wx)))
            wx = new_x
            wy = new_y

            WorldToDevice(wx, wy, pix_x, pix_y)
            If (pix_x >= 0 AndAlso pix_x < bm.Width AndAlso _
                pix_y >= 0 AndAlso pix_y < bm.Height) _
            Then
                bm.SetPixel(pix_x, pix_y, Color.Red)
            End If
        Next i
    End Sub

    ' Convert between world and device coordinates.
    Private Const Wxmin As Single = -4.0F
    Private Const Wxmax As Single = 4.0F
    Private Const Wymin As Single = -3.0F
    Private Const Wymax As Single = 3.0F
    Private Const Wwid As Single = (Wxmax - Wxmin)
    Private Const Whgt As Single = (Wymax - Wymin)
    Private Const Dxmin As Single = 0.0F
    Private Const Dxmax As Single = 400.0F
    Private Const Dymin As Single = 0.0F
    Private Const Dymax As Single = 300.0F
    Private Const Dwid As Single = (Dxmax - Dxmin)
    Private Const Dhgt As Single = (Dymax - Dymin)
    Private Sub WorldToDevice(ByVal wx As Single, ByVal wy As Single, ByRef dx As Integer, ByRef dy As Integer)
        dx = CInt((Dxmin + Dwid * (wx - Wxmin) / Wwid))
        dy = CInt((Dymin + Dhgt * (wy - Wymin) / Whgt))
    End Sub
    Private Sub DeviceToWorld(ByVal dx As Integer, ByVal dy As Integer, ByRef wx As Single, ByRef wy As Single)
        wx = Wxmin + Wwid * (dx - Dxmin) / Dwid
        wy = Wymin + Whgt * (dy - Dymin) / Dhgt
    End Sub
End Class
